/* 
 * File:   Switch_Functions.h
 * Author: rmcnamee
 *
 * Created on December 13, 2017, 10:28 AM
 */

#ifndef SWITCH_FUNCTIONS_H
#define	SWITCH_FUNCTIONS_H

#include <stdbool.h>

#define SWITCH_BRIGHTNESS_MAX_COMMAND 0x0F // Per the 64x32 datasheet
#define SWITCH_COLOR_WHITE 0xFF // Per the 64x32 datasheet
#define SWITCH_COLOR_MINIMUM 0x03 // The lowest 2 bits of the backlight color are always 1

#define SWITCH_MAX_IMAGES 0x1FF // We limit the number of images per switch type

/**
   @Param
    none
   @Returns
    none
   @Description
    Provides any setup the switches need before use
   @Example
    SetupSwitches();
 */
void SetupSwitches(void);

/**
   @Param
    none
   @Returns
    none
   @Description
    Scans both switches for being pressed, and does whatever behavior the attribute
 * specifies if the switch was pressed
   @Example
    ScanSwitchesForPress();
 */
void ScanSwitchesForPress(void);

/**
   @Param
    none
   @Returns
    none
   @Description
 Based on attributes, sets timers for switch behaviors (changing images, changing
 * backlight color, Loading new attributes, etc.)
 * 
   @Example
    CheckSwitchBehavior();
 */
void CheckSwitchBehavior(void);

/**
   @Param
 * switch1Address - The address (0001,0002) of switch 1 in the attribute table
 * switch2Address - The address (0001,0002) of switch 2 in the attribute table
   @Returns
    none
   @Description
    Manually changes the image on both of the switches
 * 
   @Example
    SetSwitchImages(0x0001,0x0002);
 */
void SetSwitchImages(unsigned short long switch1Address, unsigned short long switch2Address);

/**
   @Param
 * brightness - The brightness value to set the switch to
   @Returns
    none
   @Description
    Manually changes the brightness switch 1
 * 
   @Example
    SetSwitch1Brightness(0xFF);
 */
void SetSwitchBrightness(unsigned char brightness);

/**
   @Param
 * switchNumber - The switch on which to write the text
 * numberOfCharacters - Number of characters in the characters buffer
 * row - row of the top-left corner of the first character
 * column - column of the top-left corner of the first character
 * characters - buffer that contains the ASCII codes for the characters
   @Returns
    none
   @Description
    Adds the requested characters to the display at the specified points
 * 
   @Example
    AddCharacterToDisplay(
 */
void AddCharacterToDisplay(unsigned char switchNumber,
                            unsigned char row, unsigned char column,
                            unsigned char numberOfCharacters, unsigned char * characters);

/**
   @Param
 * lineColor - 2 byte buffer, 2 bytes of the color of the line in BGR 565 format
   @Returns
    none
   @Description
    Sets the color to be used by the CHARACTER_SUBCOMMAND_OLED1_LINE subcommand
 * 
   @Example
 * color[2] = { 0x56 0xFF };
 * SetCharacterLineColor(color);
 */
void SetCharacterLineColor(unsigned char * lineColor);

/**
   @Param
 * colorBuffer - 4 byte buffer, 2 bytes of background color and 2 bytes of foreground color in BGR 565 format
   @Returns
    none
   @Description
    Sets the color to be used by the CHARACTER_SUBCOMMAND_OLED1_ADD subcommand
 * 
   @Example
 * color[4] = { 0xFF 0xFF 0x00 0x00}; // White text black background
 * SetCharacterLineColor(color);
 */
void SetCharacterBackGroundForegroundColor(unsigned char * colorBuffer);

/**
   @Param
 * switchNumber - the switch number to add colored rows to
 * numberOfRows -the number of rows to color
 * startingRow - the starting row number to color
   @Returns
    none
   @Description
 * Colors in a number of rows on the specified switch
 * 
   @Example
 * color[4] = { 0xFF 0xFF 0x00 0x00}; // White text black background
 * SetCharacterLineColor(color);
 */
void AddLineToSwitch(unsigned char switchNumber, unsigned char numberOfRows, unsigned char startingRow);



// Switch commands per the data sheet
typedef enum
{
    SWITCH_WINDOW_COLUMN            = 0x15,
    SWITCH_DRAW_LINE                = 0x21,
    SWITCH_WINDOW_ROW               = 0x75,
    SWITCH_CONTRAST_COLOR_A         = 0x81,
    SWITCH_CONTRAST_COLOR_B         = 0x82,
    SWITCH_CONTRAST_COLOR_C         = 0x83,
    SWITCH_MASTER_CURRENT_CONTROL   = 0x87,
    SWITCH_COLOR_A_PRECHARGE_SPEED  = 0x8A,
    SWITCH_COLOR_B_PRECHARGE_SPEED  = 0x8B,
    SWITCH_COLOR_C_PRECHARGE_SPEED  = 0x8C,    
    SWITCH_COLOR_DEPTH              = 0xA0,
    SWITCH_DISPLAY_START_LINE       = 0xA1,
    SWITCH_DISPLAY_OFFSET           = 0xA2,
    SWITCH_DISPLAY_NORMAL           = 0xA4,
    SWITCH_MULTIPLEX_RATIO          = 0xA8,
    SWITCH_COLOR_DIM_MODE           = 0xAB,
    SWITCH_MASTER_CONFIGURATION     = 0xAD,
    SWITCH_POWER_SAVE_MODE          = 0xB0,
    SWITCH_PHASE_1_2_PERIOD_ADJUST  = 0xB1,
    SWITCH_DISPLAY_CLOCK_DIVIDER    = 0xB3,
    SWITCH_ENABLE_LINEAR_GRAYSCALE  = 0xB9,
    SWITCH_PRECHARGE_LEVEL          = 0xBB,
    SWITCH_SET_VCOMH                = 0xBE,
    SWITCH_DISPLAY_OFF              = 0xAE,
    SWITCH_DISPLAY_ON               = 0xAF
}switchCommandsEnum;

typedef enum
{
    ROCKER_RESET                    = 0x01,
    ROCKER_DISPLAY_ON_OFF           = 0x02,
    ROCKER_READ_WRITE_WETTING       = 0x07,
    ROCKER_WRITE_IMAGE              = 0x08,
    ROCKER_SET_DISPLAY_DIRECTION    = 0x09,
    ROCKER_RESERVED1                = 0x10,
    ROCKER_RESERVED2                = 0x12,
    ROCKER_RESERVED3                = 0x13,
    ROCKER_DISPLAY_STANDBY          = 0x14,
    ROCKER_RESERVED4                = 0x16,
    ROCKER_RESERVED5                = 0x17,
    ROCKER_RESERVED6                = 0x18,
    ROCKER_RESERVED7                = 0x1A,
    ROCKER_RESERVED8                = 0x1C,
    ROCKER_WRITE_DIRECTION          = 0x1D,
    ROCKER_COLUMN_OUPUT_RANGE       = 0x30,
    ROCKER_ROW_OUTPUT_RANGE         = 0x32,
    ROCKER_X_AXIS_START             = 0x34,
    ROCKER_X_AXIS_END               = 0x35,            
    ROCKER_Y_AXIS_START             = 0x36,
    ROCKER_Y_AXIS_END               = 0x37,
    ROCKER_X_AXIS_ADDRESS           = 0x38,
    ROCKER_Y_AXIS_ADDRESS           = 0x39,
    ROCKER_RESERVED9                = 0x48,
    ROCKER_SCREEN_SAVER_SETTING1    = 0xC3,
    ROCKER_SCREEN_SAVER_SETTING2    = 0xC4,
    ROCKER_SCREEN_SAVER_MODE        = 0xCC,
    ROCKER_SCREEN_SAVER_START_STOP  = 0xCD,
    ROCKER_CLOCK_DIVISION           = 0xD0,
    ROCKER_STBY_PIN_SETTING         = 0xD2,
    ROCKER_DACA_SETTING             = 0xD4,
    ROCKER_DACB_SETTING             = 0xD5,
    ROCKER_DACC_SETTING             = 0xD6,
    ROCKER_DACD_SETTING             = 0xD7,
    ROCKER_BRIGHTNESS               = 0xD9,
    ROCKER_DIMMER_SETTING           = 0xDB,
    ROCKER_RESERVED11               = 0xDD,            
            
}rockerCommandsEnum;

#define FRAMELESS_CONTRAST_A_DEFAULT            0x1C
#define FRAMELESS_CONTRAST_B_DEFAULT            0x2E
#define FRAMELESS_CONTRAST_C_DEFAULT            0x2C
#define FRAMELESS_COLOR_A_PRECHARGE_DEFAULT     0x4F
#define FRAMELESS_COLOR_B_PRECHARGE_DEFAULT     0x74
#define FRAMELESS_COLOR_C_PRECHARGE_DEFAULT     0x88
#define FRAMELESS_MASTER_CURRENT_DEFAULT        0x0F
#define FRAMELESS_COLOR_DEPTH_DEFAULT           0x68
#define FRAMELESS_DISPLAY_START_LINE_DEFAULT    0x00
#define FRAMELESS_DISPLAY_OFFSET_DEFAULT        0x00
#define FRAMELESS_MULTIPLEX_RATIO_DEFAULT       0x3F
#define FRAMELESS_DIM_MODE_DEFAULT_1            0x00
#define FRAMELESS_DIM_MODE_DEFAULT_2            0x0E
#define FRAMELESS_DIM_MODE_DEFAULT_3            0x17
#define FRAMELESS_DIM_MODE_DEFAULT_4            0x16
#define FRAMELESS_DIM_MODE_DEFAULT_5            0x12
#define FRAMELESS_MASTER_CONFIGURATION_DEFAULT  0x8E
#define FRAMELESS_POWER_SAVE_MODE_DEFAULT       0x0B
#define FRAMELESS_PHASE_PERIOD_DEFAULT          0x17
#define FRAMELESS_DISPLAY_CLOCK_DIVIDER_DEFAULT 0xF0
#define FRAMELESS_PRECHARGE_LEVEL_DEFAULT       0x1D
#define FRAMELESS_VCOMH_DEFAULT                 0x20
#define FRAMELESS_WINDOW_COLUMN_DEFAULT_1       0x00
#define FRAMELESS_WINDOW_COLUMN_DEFAULT_2       0x5F
#define FRAMELESS_WINDOW_ROW_DEFAULT_1          0x00
#define FRAMELESS_WINDOW_ROW_DEFAULT_2          0x3F

#define STANDARD_CONTRAST_A_DEFAULT             0x15
#define STANDARD_CONTRAST_B_DEFAULT             0x1A
#define STANDARD_CONTRAST_C_DEFAULT             0x17
#define STANDARD_MASTER_CURRENT_DEFAULT         0x0F
#define STANDARD_COLOR_DEPTH_DEFAULT            0x70
#define STANDARD_DISPLAY_START_LINE_DEFAULT     0x00
#define STANDARD_DISPLAY_OFFSET_DEFAULT         0x10
#define STANDARD_MULTIPLEX_RATIO_DEFAULT        0x2F
#define STANDARD_DIM_MODE_DEFAULT_1             0x00
#define STANDARD_DIM_MODE_DEFAULT_2             0x12
#define STANDARD_DIM_MODE_DEFAULT_3             0x0C
#define STANDARD_DIM_MODE_DEFAULT_4             0x14
#define STANDARD_DIM_MODE_DEFAULT_5             0x12
#define STANDARD_MASTER_CONFIGURATION_DEFAULT   0x8E
#define STANDARD_POWER_SAVE_MODE_DEFAULT        0x0B
#define STANDARD_PHASE_PERIOD_DEFAULT           0x44
#define STANDARD_DISPLAY_CLOCK_DIVIDER_DEFAULT  0xA0
#define STANDARD_PRECHARGE_LEVEL_DEFAULT        0xB9
#define STANDARD_VCOMH_DEFAULT                  0x3E
#define STANDARD_WINDOW_COLUMN_DEFAULT_1        0x10
#define STANDARD_WINDOW_COLUMN_DEFAULT_2        0x4F
#define STANDARD_WINDOW_ROW_DEFAULT_1           0x00
#define STANDARD_WINDOW_ROW_DEFAULT_2           0x2F

#define DISPLAY_CONTRAST_A_DEFAULT              0x0E
#define DISPLAY_CONTRAST_B_DEFAULT              0x13
#define DISPLAY_CONTRAST_C_DEFAULT              0x12
#define DISPLAY_MASTER_CURRENT_DEFAULT          0x0F
#define DISPLAY_COLOR_DEPTH_DEFAULT             0x70
#define DISPLAY_DISPLAY_START_LINE_DEFAULT      0x00
#define DISPLAY_DISPLAY_OFFSET_DEFAULT          0x1C
#define DISPLAY_MULTIPLEX_RATIO_DEFAULT         0x23
#define DISPLAY_DIM_MODE_DEFAULT_1              0x00
#define DISPLAY_DIM_MODE_DEFAULT_2              0x07
#define DISPLAY_DIM_MODE_DEFAULT_3              0x0A
#define DISPLAY_DIM_MODE_DEFAULT_4              0x09
#define DISPLAY_DIM_MODE_DEFAULT_5              0x12
#define DISPLAY_MASTER_CONFIGURATION_DEFAULT    0x8E
#define DISPLAY_POWER_SAVE_MODE_DEFAULT         0x0B
#define DISPLAY_PHASE_PERIOD_DEFAULT            0x44
#define DISPLAY_DISPLAY_CLOCK_DIVIDER_DEFAULT   0x30
#define DISPLAY_PRECHARGE_LEVEL_DEFAULT         0x12
#define DISPLAY_VCOMH_DEFAULT                   0x3C
#define DISPLAY_WINDOW_COLUMN_DEFAULT_1         0x16
#define DISPLAY_WINDOW_COLUMN_DEFAULT_2         0x49
#define DISPLAY_WINDOW_ROW_DEFAULT_1            0x00
#define DISPLAY_WINDOW_ROW_DEFAULT_2            0x23

#define ROCKER_DISPLAY_OFF_DEFAULT              0x00
#define ROCKER_READ_WRITE_WETTING_DEFAULT       0x00
#define ROCKER_DISPLAY_DIRECTION_DEFAULT        0x00
#define ROCKER_RESERVED1_DEFAULT                0x03
#define ROCKER_RESERVED2_DEFAULT                0x63
#define ROCKER_RESERVED3_DEFAULT                0x00
#define ROCKER_DISPLAY_STANDBY_DEFAULT          0x00
#define ROCKER_RESERVED4_DEFAULT                0x00
#define ROCKER_RESERVED5_DEFAULT                0x00
#define ROCKER_RESERVED6_DEFAULT                0x09
#define ROCKER_RESERVED7_DEFAULT                0x04
#define ROCKER_RESERVED8_DEFAULT                0x00
#define ROCKER_WRITE_DIRECTION_DEFAULT          0x00
#define ROCKER_COLUMN_OUTPUT_RANGE_DEFAULT1     0x00
#define ROCKER_COLUMN_OUTPUT_RANGE_DEFAULT2     0x5F
#define ROCKER_ROW_OUTPUT_RANGE_DEFAULT1        0x00
#define ROCKER_ROW_OUTPUT_RANGE_DEFAULT2        0x3F
#define ROCKER_X_AXIS_START_DEFAULT             0x00
#define ROCKER_X_AXIS_END_DEFAULT               0x0B
#define ROCKER_Y_AXIS_START_DEFAULT             0x00
#define ROCKER_Y_AXIS_END_DEFAULT               0x3F
#define ROCKER_X_AXIS_ADDRESS_DEFAULT           0x00
#define ROCKER_Y_AXIS_ADDRESS_DEFAULT           0x00
#define ROCKER_RESERVED9_DEFAULT                0x03
#define ROCKER_SCREEN_SAVER_SETTING1_DEFAULT    0x00
#define ROCKER_SCREEN_SAVER_SETTING2_DEFAULT    0x00
#define ROCKER_SCREEN_SAVER_MODE_DEFAULT        0x00
#define ROCKER_SCREEN_SAVER_STOP_DEFAULT        0x00
#define ROCKER_CLOCK_DIVISION_DEFAULT           0x80
#define ROCKER_STBY_PIN_SETTING_DEFAULT         0x00
#define ROCKER_DACA_SETTING_DEFAULT             0x00
#define ROCKER_DACB_SETTING_DEFAULT             0x00
#define ROCKER_DACC_SETTING_DEFAULT             0x00
#define ROCKER_DACD_SETTING_DEFAULT             0x00
#define ROCKER_BRIGHTNESS_DEFAULT               0x00
#define ROCKER_DIMMER_SETTING_DEFAULT           0x0F
#define ROCKER_RESERVED11_DEFAULT               0x88
#define ROCKER_DISPLAY_ON_PARAMETER             0x01


// A description of every byte of the attribute block
typedef enum
{
    ATTRIBUTE_CURRENT_ADDRESS_HIGH = 0x00, // The current location
    ATTRIBUTE_CURRENT_ADDRESS_LOW,
    ATTRIBUTE_LOOP_END_ADDRESS_HIGH, // If looping, the last number of the loop (003, 0004, etc.)
    ATTRIBUTE_LOOP_END_ADDRESS_LOW,
    ATTRIBUTE_TIMER1, // Time between images of a loop, if high byte is 0 it's not a loop
    ATTRIBUTE_TIMER2,
    ATTRIBUTE_SW1_PRESS_ADDRESS_HIGH, // Address to jump to if switch 1 is pressed
    ATTRIBUTE_SW1_PRESS_ADDRESS_LOW,
    ATTRIBUTE_SW2_PRESS_ADDRESS_HIGH, // Address to jump to if switch 2 is pressed
    ATTRIBUTE_SW2_PRESS_ADDRESS_LOW,
    ATTRIBUTE_SW1_TIMER_EXPIRED_JUMP_ADDRESS_HIGH, // Address to jump to after a loop completes
    ATTRIBUTE_SW1_TIMER_EXPIRED_JUMP_ADDRESS_LOW,
    ATTRIBUTE_SW2_TIMER_EXPIRED_JUMP_ADDRESS_HIGH, // Address to jump to after a loop completes
    ATTRIBUTE_SW2_TIMER_EXPIRED_JUMP_ADDRESS_LOW,
    ATTRIBUTE_RESERVED1, // Reserved for future functionality
    ATTRIBUTE_RESERVED2 // Reserved for future functionality
            
}attributeByteDescriptionsEnum;

typedef enum
{
    SETUP_SETUP_ADDRESS_HIGH = 0x00,
    SETUP_SETUP_ADDRESS_LOW,
    SETUP_SWITCH2_START_ADDRESS_HIGH,
    SETUP_SWITCH2_START_ADDRESS_LOW,
    SETUP_BRIGHTNESS,
    SETUP_RESERVED6,
    SETUP_RESERVED7,
    SETUP_RESERVED8,
    SETUP_RESERVED9,
    SETUP_RESERVED10,
    SETUP_RESERVED11,
    SETUP_RESERVED12,
    SETUP_RESERVED13,
    SETUP_RESERVED14,
    SETUP_RESERVED15,
    SETUP_RESERVED16
}setupByteDescriptionsEnum;

typedef enum
{
    ROCKER_ATTRIBUTE_CURRENT_ADDRESS_HIGH = 0x00, // The current location
    ROCKER_ATTRIBUTE_CURRENT_ADDRESS_LOW,
    ROCKER_ATTRIBUTE_LOOP_END_ADDRESS_HIGH, // If looping, the last number of the loop (003, 0004, etc.)
    ROCKER_ATTRIBUTE_LOOP_END_ADDRESS_LOW,
    ROCKER_ATTRIBUTE_TIMER1, // Time between images of a loop, if high byte is 0 it's not a loop
    ROCKER_ATTRIBUTE_TIMER2,
    ROCKER_ATTRIBUTE_SW1_PRESS_ADDRESS_HIGH, // Address to jump to if switch 1 is pressed
    ROCKER_ATTRIBUTE_SW1_PRESS_ADDRESS_LOW,
    ROCKER_ATTRIBUTE_SW2_PRESS_ADDRESS_HIGH, // Address to jump to if switch 2 is pressed
    ROCKER_ATTRIBUTE_SW2_PRESS_ADDRESS_LOW,
    ROCKER_ATTRIBUTE_SW3_PRESS_ADDRESS_HIGH, // Address to jump to if switch 2 is pressed
    ROCKER_ATTRIBUTE_SW3_PRESS_ADDRESS_LOW,
    ROCKER_ATTRIBUTE_TIMER_EXPIRED_JUMP_ADDRESS_HIGH, // Address to jump to after a loop completes
    ROCKER_ATTRIBUTE_TIMER_EXPIRED_JUMP_ADDRESS_LOW,
    ROCKER_ATTRIBUTE_RESERVED1, // Reserved for future functionality
    ROCKER_ATTRIBUTE_RESERVED2 // Reserved for future functionality
}rockerattributeByteDescriptionsEnum;

// A struct to contain the current switch settings based on the loaded attribute
typedef struct
{
    unsigned short long currentAddress;
    unsigned short long loopEndAddress;
    unsigned short long loopCurrentAddress;
    unsigned char timer1;
    unsigned char timer2;
    unsigned short long switch1PressAddress;
    unsigned short long switch2PressAddress;
    unsigned short long rockerTopPressAddress;
    unsigned short long rockerMiddlePressAddress;
    unsigned short long rockerBottomPressAddress;
    unsigned short long switch1TimerExpiredJumpAddress;
    unsigned short long switch2TimerExpiredJumpAddress;
    unsigned char reserved1;
    unsigned char reserved2;
}imageAttributeStruct;

typedef enum
{
    DEV_KIT_TWO_FRAMELESS = 0,
    DEV_KIT_ONE_FRAMELESS_ONE_STANDARD,
    DEV_KIT_ONE_STANDARD_ONE_DISPLAY,
    DEV_KIT_ROCKER,
    DEV_KIT_UNASSIGNED = 0xFF
}devKitTypeEnum;

#endif	/* SWITCH_FUNCTIONS_H */

